"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Router = void 0;
const index_1 = require("../../index");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const lodash_1 = require("lodash");
const express_http_proxy_1 = __importDefault(require("express-http-proxy"));
class Router {
    constructor(routes, proxies, routesMiddleware = [], basePath) {
        this.routes = routes;
        this.proxies = proxies;
        this.server = index_1.server;
        this.basePath = basePath;
        this.routesMiddleware = routesMiddleware;
        this.load();
    }
    // Starts loading the routes
    load() {
        this.createRoute();
        this.createRouteProxy();
        // Replace the default error handler (This avoid the application to crash)
        this.errorHandling();
    }
    createRouteProxy() {
        if (this.proxies) {
            this.proxies.forEach((proxy) => {
                index_1.logger.debug(`Loading proxy ${proxy.path}:${proxy.host}`, 'ihf.component.loading.proxy');
                this.createRoutesMiddleware();
                const itemProxy = express_http_proxy_1.default(proxy.host);
                this.server.app.use(proxy.path, this.privateRouteMiddleware(proxy.private, proxy.acl), (req, res, next) => {
                    itemProxy(req, res, next);
                });
            });
        }
    }
    createRoute() {
        if (this.routes) {
            this.routes.forEach((route) => {
                let routePath = `/${this.basePath}${route.path.indexOf('/') === 0 ? route.path : `/${route.path}`}`;
                // If is a root component
                if (this.basePath === 'root') {
                    routePath =
                        `${route.path.indexOf('/') === 0 ? route.path : `/${route.path}`}`;
                }
                const { controller, method = 'use', middlewares = [], } = route;
                index_1.logger.debug(`Loading route ${method}: "${routePath}"`, 'ihf.component.loading.route');
                const routeMiddlewares = [
                    this.privateRouteMiddleware(route.private, route.acl),
                    ...this.routesMiddleware,
                    ...middlewares,
                ];
                this.server.app[method](routePath, ...routeMiddlewares, controller);
            });
        }
    }
    createRoutesMiddleware() {
        if (!this.routesMiddleware && this.routesMiddleware.length === 0) {
            return (req, res, next) => next();
        }
        return this.routesMiddleware;
    }
    privateRouteMiddleware(privateRoute, acl) {
        // Public route
        if (!privateRoute)
            return (req, res, next) => next();
        // Private route
        // JWT Middleware
        const _403 = (res) => res.status(403).json({
            error: {
                code: 403,
                message: 'Not authorized',
            },
        });
        const _401 = (res) => res.status(401).json({
            error: {
                code: 401,
                message: 'Invalid token',
            },
        });
        return (req, res, next) => {
            // Check if authorization is set and if it's Bearer
            const { authorization } = req.headers;
            if (!authorization || authorization.indexOf('Bearer ') === -1)
                return _401(res);
            const token = authorization.replace('Bearer ', '');
            if (token.length === 0)
                return _401(res);
            const jwtSecret = this.server.app.get('jwt_secret');
            jsonwebtoken_1.default.verify(token, jwtSecret, (error, decodedToken) => {
                if (error) {
                    return _401(res);
                }
                // Pass the token/user information in the request object
                req.user = decodedToken;
                // Register user context to APM
                if (index_1.apm) {
                    index_1.apm.setUserContext({
                        id: decodedToken.id || null,
                        username: decodedToken.username || decodedToken.email || null,
                        email: decodedToken.email || null,
                    });
                }
                // Check the ACL
                if (acl && Array.isArray(acl)) { // Check if valid acl object was passed
                    // Check if the token has the prop setted as acl property
                    if (!decodedToken.acl || decodedToken.acl.length === 0) {
                        return _403(res);
                    }
                    // If the user is root, let him in!
                    if (decodedToken.acl.includes('Root'))
                        return next();
                    // Check for positive acl
                    const positivesAcl = acl.filter((p) => p.indexOf('!') !== 0);
                    if (positivesAcl.length > 0) {
                        const matchedAcl = lodash_1.intersection(positivesAcl, decodedToken.acl);
                        if (matchedAcl.length === 0) {
                            return _403(res);
                        }
                    }
                    // Check for negative acl
                    const negativesAcl = acl.filter((p) => p.indexOf('!') === 0);
                    if (negativesAcl.length > 0) {
                        const matchedAcl = lodash_1.intersection(negativesAcl.map((p) => p.replace('!', '')), decodedToken.acl);
                        if (matchedAcl.length > 0) {
                            return _403(res);
                        }
                    }
                    return next();
                }
                next();
            });
        };
    }
    errorHandling() {
        this.server.app.use((err, req, res, next) => {
            index_1.logger.error(`Uncaught error on ${req.method}:${req.path} - ${err.message || err}`, 'ihf.component.route.uncaughtError');
            next();
        });
    }
}
exports.Router = Router;
//# sourceMappingURL=index.js.map